@@ -1,13 +1,14 @@
 using Documenter, IterTools
 
+DocMeta.setdocmeta!(IterTools, :DocTestSetup, :(using IterTools))
 makedocs(
     modules = [IterTools],
     sitename = "IterTools",
     pages = [
-        "Introduction" => "index.md",
-        "Function index" => "functionindex.md"
+        "Home" => "index.md",
+        "API Reference" => "reference.md"
         ],
-    doctest=false,
+    doctest=true,
    )
 
 deploydocs(
