@@ -265,14 +265,13 @@ end
 #   partition(count(1), 2, 1) = (1,2), (2,3), (3,4) ...
 #   partition(count(1), 2, 3) = (1,2), (4,5), (7,8) ...
 
-struct Partition{I, N}
+struct Partition{I, N, K}
     xs::I
-    step::Int
 end
 _length_partition(l, n, s) = ifelse(l - n ≥ 0, ((l - n) ÷ s) + 1, 0)
-eltype(::Type{Partition{I, N}}) where {I, N} = NTuple{N, eltype(I)}
-length(it::Partition{I, N}) where {I, N} = _length_partition(length(it.xs), N, it.step)
-IteratorSize(::Type{Partition{I, N}}) where {I, N} = longest(HasLength(), IteratorSize(I))
+eltype(::Type{Partition{I, N, K}}) where {I, N, K} = NTuple{N, eltype(I)}
+length(it::Partition{I, N, K}) where {I, N, K} = _length_partition(length(it.xs), N, K)
+IteratorSize(::Type{Partition{I, N, K}}) where {I, N, K} = longest(HasLength(), IteratorSize(I))
 
 """
     partition(xs, n, [step])
@@ -320,47 +319,46 @@ i = (7, 8)
     if step < 1
         throw(ArgumentError("Partition step must be at least 1."))
     end
-
     if n < 1
-        throw(ArgumentError("Partition size n must be at least 1"))
+        throw(ArgumentError("Partition size n must be at least 1."))
     end
 
-    Partition{I, n}(xs, step)
+    Partition{I, n, step}(xs)
 end
 
-function iterate(it::Partition{I, N}, state=nothing) where {I, N}
-    if state === nothing
-        result = Vector{eltype(I)}(undef, N)
+@generated function iterate(it::Partition{I, N, K}) where {I, N, K}
+    res = Expr(:block)
+    tup = Expr(:tuple)
+    var = gensym()
+    push!(res.args, :(($var, xs_state) = @ifsomething iterate(it.xs)))
+    push!(tup.args, var)
+    for i in 2:N
+        var = gensym()
+        push!(res.args, :(($var, xs_state) = @ifsomething iterate(it.xs, xs_state)))
+        push!(tup.args, var)
+    end
+    quote
+        $res
+        newels = $tup
+        (newels, (newels[$K+1:end], xs_state))
+    end
+end
 
-        result[1], xs_state = @ifsomething iterate(it.xs)
-
-        for i in 2:N
-            result[i], xs_state = @ifsomething iterate(it.xs, xs_state)
+@generated function iterate(it::Partition{I, N, K}, (els, xs_state)) where {I, N, K}
+    res = Expr(:block)
+    tup = Expr(:tuple, :(els...))
+    for i in 1:K
+        var = gensym()
+        push!(res.args, :(($var, xs_state) = @ifsomething iterate(it.xs, xs_state)))
+        if i > K-N
+            push!(tup.args, var)
         end
-    else
-        (xs_state, result) = state
-        result[end], xs_state = @ifsomething iterate(it.xs, xs_state)
     end
-
-    p = similar(result)
-    overlap = max(0, N - it.step)
-    p[1:overlap] .= result[it.step .+ (1:overlap)]
-
-    # when step > n, skip over some elements
-    for i in 1:max(0, it.step - N)
-        xs_iter = iterate(it.xs, xs_state)
-        xs_iter === nothing && break
-        _, xs_state = xs_iter
+    quote
+        $res
+        newels = $tup
+        (newels, (newels[$K+1:end], xs_state))
     end
-
-    for i in (overlap + 1):(N - 1)
-        xs_iter = iterate(it.xs, xs_state)
-        xs_iter === nothing && break
-
-        p[i], xs_state = xs_iter
-    end
-
-    return (tuple(result...)::eltype(Partition{I, N}), (xs_state, p))
 end
 
 
@@ -550,7 +548,7 @@ IteratorEltype(::Type{Binomial{C}}) where {C} = IteratorEltype(C)
 
 subsets(xs, k) = Binomial(xs, length(xs), k)
 
-mutable struct BinomialIterState
+struct BinomialIterState
     idx::Vector{Int64}
     done::Bool
 end
@@ -575,9 +573,7 @@ function iterate(it::Binomial, state=BinomialIterState(collect(Int64, 1:it.k), i
         end
     end
 
-    state.done = i == 0
-
-    return set, state
+    return set, BinomialIterState(idx, i == 0)
 end
 
 
@@ -597,7 +593,7 @@ subsets(xs::C, ::Val{K}) where {K, C} = StaticSizeBinomial{K, C}(xs)
 iterate(it::StaticSizeBinomial{0}, state=false) = state ? nothing : ((), true)
 
 # Generic case K >= 1
-pop(t::NTuple) = reverse(tail(reverse(t))), t[end]
+pop(t::NTuple) = Base.front(t), last(t)
 
 function advance(it::StaticSizeBinomial{K}, idx) where {K}
     xs = it.xs
@@ -1108,12 +1104,12 @@ Input:
 ```jldoctest
 julia> collect(interleaveby(1:2:5, 2:2:6))
 6-element Vector{Int64}:
-  1
-  2
-  3
-  4
-  5
-  6
+ 1
+ 2
+ 3
+ 4
+ 5
+ 6
 ```
 
 If the predicate is `Base.isless` (the default) and both inputs are sorted, this produces the sorted output.
@@ -1217,6 +1213,25 @@ _zip_longest_promote_shape(a, b) = promote_shape(a, b)
 For one or more iterable objects, return an iterable of tuples, where the `i`th tuple
 contains the `i`th component of each input iterable if it is not finished, and `default`
 otherwise. `default` can be a scalar, or a tuple with one default per iterable.
+
+```jldoctest
+julia> for t in zip_longest(1:2, 5:8)
+         @show t
+       end
+t = (1, 5)
+t = (2, 6)
+t = (nothing, 7)
+t = (nothing, 8)
+
+julia> for t in zip_longest('a':'e', ['m', 'n']; default='x')
+         @show t
+       end
+t = ('a', 'm')
+t = ('b', 'n')
+t = ('c', 'x')
+t = ('d', 'x')
+t = ('e', 'x')
+```
 """
 zip_longest(its...; default=nothing) = ZipLongest(Tuple(_Padded.(its, default)))
 
